@@ -12,12 +12,45 @@ from .ciphers import TripleDes, AesCbc
 from .utils import wrap_lines
 
 
+class UnsupportedAlgorithmError(Exception):
+    """
+    An exception indicating that an unsupported cipher algorithm was specified
+    """
+
+    pass
+
+
+def _get_content_encryption_algorithm(alg):
+    algs = {
+        "tripledes_3key": TripleDes(alg, key_size=24),
+        "aes128_cbc": AesCbc(alg, key_size=16),
+        "aes256_cbc": AesCbc(alg, key_size=32)
+    }
+    try:
+        return algs[alg]
+    except KeyError:
+        raise UnsupportedAlgorithmError("selected algorithm \"{}\" not in: "
+                                        "{}".format(alg, ", ".join(algs.keys())))
+
+
+def _get_key_encryption_algorithm(alg):
+    algs = {
+        "rsaes_pkcs1v15": asymmetric.rsa_pkcs1v15_encrypt,
+        "rsa": asymmetric.rsa_pkcs1v15_encrypt  # rsa is mapped to rsaes_pkcs1v15 in asn1crypto
+    }
+    try:
+        return algs[alg]
+    except KeyError:
+        raise UnsupportedAlgorithmError("selected algorithm \"{}\" not in: "
+                                        "{}".format(alg, ", ".join(algs.keys())))
+
+
 def _iterate_recipient_infos(certs, session_key, key_enc_alg):
     """Yields the recipient identifier data needed for an encrypted message.
 
     Args:
         certs (:obj:`list` of :obj:`oscrypto.asymmetric.Certificate`): Certificate object
-        session_key (str): Session key
+        session_key (bytes): Session key
         key_enc_alg (str): Key Encryption Algorithm
 
     Yields:
@@ -34,7 +67,7 @@ def get_recipient_info_for_cert(cert, session_key, key_enc_alg="rsaes_pkcs1v15")
 
     Args:
         cert (:obj:`oscrypto.asymmetric.Certificate`): Certificate object
-        session_key (str): Session key
+        session_key (bytes): Session key
         key_enc_alg (str): Key Encryption Algorithm
 
     Returns:
@@ -52,10 +85,8 @@ def get_recipient_info_for_cert(cert, session_key, key_enc_alg="rsaes_pkcs1v15")
 
     serial = cert.asn1['tbs_certificate']['serial_number'].native
 
-    if key_enc_alg in ["rsa", "rsaes_pkcs1v15"]:  # rsa is mapped to rsaes_pkcs1v15 in asn1crypto
-        encrypted_key = asymmetric.rsa_pkcs1v15_encrypt(cert.public_key, session_key)
-    else:
-        raise NotImplementedError("Unsupported Key Encryption Algorithm")
+    key_enc_func = _get_key_encryption_algorithm(key_enc_alg)
+    encrypted_key = key_enc_func(cert.public_key, session_key)
 
     return cms.RecipientInfo(
         name="ktri",
@@ -77,15 +108,19 @@ def get_recipient_info_for_cert(cert, session_key, key_enc_alg="rsaes_pkcs1v15")
     )
 
 
-def encrypt_message(message, certs_recipients, content_enc_alg="aes256_cbc", key_enc_alg="rsaes_pkcs1v15", prefix=""):
+def encrypt_message(message, certs_recipients,
+                    content_enc_alg="aes256_cbc", key_enc_alg="rsaes_pkcs1v15", prefix=""):
     """Takes a message and returns a new message with the original content as encrypted body
 
-    Take the contents of the message parameter, formatted as in RFC 2822 (type bytes, str or message)
-    and encrypts them, so that they can only be read by the intended recipient specified by pubkey.
+    Take the contents of the message parameter, formatted as in RFC 2822 (type bytes, str or
+        message) and encrypts them, so that they can only be read by the intended recipient
+        specified by pubkey.
 
     Args:
         message (bytes, str or :obj:`email.message.Message`): Message to be encrypted.
-        certs_recipients (:obj:`list` of :obj:`oscrypto.asymmetric.Certificate`):
+        certs_recipients (:obj:`list` of `bytes`, `str` or :obj:`asn1crypto.x509.Certificate` or
+            :obj:`oscrypto.asymmetric.Certificate): A list of byte string of file contents, a
+            unicode string filename or an asn1crypto.x509.Certificate object
         key_enc_alg (str): Key Encryption Algorithm
         content_enc_alg (str): Content Encryption Algorithm
         prefix (str): Content type prefix (e.g. "x-"). Default to ""
@@ -98,20 +133,16 @@ def encrypt_message(message, certs_recipients, content_enc_alg="aes256_cbc", key
 
     """
 
-    # Get the chosen block cipher
-    if content_enc_alg == "tripledes_3key":
-        block_cipher = TripleDes(content_enc_alg, key_size=24)
-    elif content_enc_alg == "aes128_cbc":
-        block_cipher = AesCbc(content_enc_alg, key_size=16)
-    elif content_enc_alg == "aes256_cbc":
-        block_cipher = AesCbc(content_enc_alg, key_size=32)
-    else:
-        raise ValueError("Unknown block algorithm")
+    certificates = []
+    for cert in certs_recipients:
+        if isinstance(cert, asymmetric.Certificate):
+            certificates.append(cert)
+        else:
+            certificates.append(asymmetric.load_certificate(cert))
 
-    if key_enc_alg == "rsaes_pkcs1v15":
-        pass
-    else:
-        raise ValueError("Unknown block algorithm")
+    # Check/Get the chosen algorithms for content and key encryption
+    block_cipher = _get_content_encryption_algorithm(content_enc_alg)
+    _ = _get_key_encryption_algorithm(key_enc_alg)
 
     # Get the message content. This could be a string, bytes or a message object
     passed_as_str = isinstance(message, str)
@@ -141,7 +172,7 @@ def encrypt_message(message, certs_recipients, content_enc_alg="aes256_cbc", key
     content = copied_msg.as_string()
     recipient_infos = []
 
-    for recipient_info in _iterate_recipient_infos(certs_recipients, block_cipher.session_key, key_enc_alg=key_enc_alg):
+    for recipient_info in _iterate_recipient_infos(certificates, block_cipher.session_key, key_enc_alg=key_enc_alg):
         if recipient_info is None:
             raise ValueError("Unknown public-key algorithm")
         recipient_infos.append(recipient_info)
