@@ -2,6 +2,9 @@ from copy import deepcopy
 from email import message_from_bytes, message_from_string
 from email.header import Header
 
+from asn1crypto import x509
+from oscrypto import asymmetric
+
 from smail.encrypt import encrypt_message
 from smail.sign import sign_message
 
@@ -54,12 +57,16 @@ def sign_and_encrypt_message(message, key_signer, cert_signer, certs_recipients,
 
     Args:
         message (:obj:`email.message.Message`): The message object to sign and encrypt.
-        key_signer (:obj:`asn1crypto.keys.PrivateKeyInfo`): Private key used to sign the
-            message.
-        cert_signer (:obj:`asn1crypto.x509.Certificate`): Certificate/Public Key
-            (belonging to Private Key) that will be included in the signed message.
-        certs_recipients (:obj:`list` of :obj:`asn1crypto.x509.Certificate`): List of
-            certificates for which the message should be encrypted.
+
+        key_signer (`bytes`, `str` or :obj:`asn1crypto.keys.PrivateKeyInfo`): Private key used to
+            sign the message. (A byte string of file contents, a unicode string filename or an
+            asn1crypto.keys.PrivateKeyInfo object)
+        cert_signer (`bytes`, `str` or :obj:`asn1crypto.x509.Certificate`): Certificate/Public Key
+            (belonging to Private Key) that will be included in the signed message. (A byte string of file
+            contents, a unicode string filename or an asn1crypto.x509.Certificate object)
+        certs_recipients (:obj:`list` of `bytes`, `str` or :obj:`asn1crypto.x509.Certificate`): A
+            list of byte string of file contents, a unicode string filename or an
+            asn1crypto.x509.Certificate object for which the message should be encrypted.
         digest_alg (str): Digest (Hash) Algorithm - e.g. "sha256"
         sig_alg (str): Signature Algorithm
         attrs (bool): Whether to include signed attributes (signing time). Default
@@ -86,6 +93,19 @@ def sign_and_encrypt_message(message, key_signer, cert_signer, certs_recipients,
     if passed_as_bytes:
         message = message_from_bytes(message)
 
+    # private key
+    key_signer = asymmetric.load_private_key(key_signer)
+    if not isinstance(key_signer, asymmetric.PrivateKey):
+        raise AttributeError("only oscrypto.asymmetric.PrivateKey supported here now")
+
+    # cert
+    cert_signer_oscrypto = asymmetric.load_certificate(cert_signer)
+    cert_signer = cert_signer_oscrypto.asn1
+    if not isinstance(cert_signer, x509.Certificate):
+        raise AttributeError("only asn1crypto.x509.Certificate supported here now")
+
+    certs_recipients = [asymmetric.load_certificate(x) for x in certs_recipients]
+
     # Extract the message payload without conversion, & the outermost MIME header / Content headers. This allows
     # the MIME content to be rendered for any outermost MIME type incl. multipart
     copied_msg = deepcopy(message)
